#include <cassert>

// Include xml-schema.hxx instead of xml-schema-custom.hxx here.
//
#include "xs/xml-schema.hxx"

#include <xercesc/dom/DOMComment.hpp>
#include <xercesc/dom/DOMDocument.hpp>

#include <xsd/cxx/xml/string.hxx> // xml::transcode, xml::string

namespace xml = xsd::cxx::xml;

namespace xml_schema
{
  type::
  type ()
      : type_base ()
  {
  }

  type::
  type (const xercesc::DOMElement& e, flags f, container* c)
      : type_base (e, f, c)
  {
    // Extract element name and namespace.
    //
    name_ = xml::transcode<char> (e.getLocalName ());

    if (const XMLCh* ns = e.getNamespaceURI ())
      namespace_ = xml::transcode<char> (ns);
  }

  type::
  type (const xercesc::DOMAttr& a, flags f, container* c)
      : type_base (a, f, c)
  {
    // No substitution groups in attributes.
    //
  }

  type::
  type (const std::string& s, const xercesc::DOMElement* e,
        flags f, container* c)
      : type_base (s, e, f, c)
  {
    // No substitution groups in list items.
    //
  }

  type::
  type (const type& x, flags f, container* c)
      : type_base (x, f, c), name_ (x.name_), namespace_ (x.namespace_)
  {
  }

  type* type::
  _clone (flags f, container* c) const
  {
    return new type (*this, f, c);
  }

  // Serialization operators.
  //
  void
  operator<< (xercesc::DOMElement& e, const type& x)
  {
    using namespace std;
    using namespace xercesc;

    // Change element name if it doesn't match name_.
    //
    const string& n (x._name ());
    const string& ns (x._namespace ());

    if (!n.empty () &&
        (xml::transcode<char> (e.getLocalName ()) != n ||
         xml::transcode<char> (e.getNamespaceURI ()) != ns))
    {
      string qn;

      if (!ns.empty ())
      {
        xml::string xns (ns);

#if _XERCES_VERSION >= 30000
        const XMLCh* p (e.lookupPrefix (xns.c_str ()));
#else
        const XMLCh* p (e.lookupNamespacePrefix (xns.c_str (), false));
#endif

        if (p == 0)
        {
          if (!e.isDefaultNamespace (xns.c_str ()))
            assert (false); // No prefix for this namespace.
        }
        else
          qn = xml::transcode<char> (p);

        if (!qn.empty ())
          qn += ':';
      }

      qn += n;

      DOMDocument* doc = e.getOwnerDocument ();
      DOMNode* n = doc->renameNode (
        &e,
        ns.empty () ? 0 : xml::string (ns).c_str (),
        xml::string (qn).c_str ());

      assert (n == &e);
    }

    // Call our base.
    //
    const type_base& b (x);
    e << b;
  }
}
